//
//  PPMDateCell.m
//  PPMDateCell
//
//  Created by ppm on 07/07/01.
//  Copyright 2007 ppm. All rights reserved.
//

#import "PPMDateCell.h"

//日付左右のマージン(の、片側分)
#define HORIZONTAL_MARGIN 2

@implementation PPMDateCell

- (PPMDateCell *) initWithFontSize:(float)size parenticView:(NSView *)view {

	self = [super init];

	fontSize = size; //「他のカラムのフォントサイズと同じ」とできればよりスマートだが、やり方が分からないので。
	parenticView = [[view retain] autorelease]; //親的なビュー。これもただ単に、セルを含んでいるビューにフォーカスが当たっているかを、セル内から判定する方法が分からないため。
	
	return self;
	
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView {
	
	NSDate *date = [self objectValue];
	
	if ( !date ) return;
	
	// 上記の日付そのものの長さで表示できるかを判定すると、セルごとに表示がバラバラになってしまうので、一定の（十分に長い）日付で判定する作戦。
	// でも、毎回計算する必要はないだろう。初期化のときに計算して、保持しておくべき？
	
	NSDate *testDate = [NSDate dateWithString:@"9999-12-31 23:59:59 +0000"];
	
	NSDateFormatter *formatter = [self formatter];
	
	if ( formatter == nil ) { 
	
		formatter = [[[NSDateFormatter alloc] init] autorelease];
		
		[formatter setFormatterBehavior:NSDateFormatterBehavior10_4];
		
		[self setFormatter:formatter];
		
	}
	
	
	/* ---- 表示する文字列のアトリビュートを設定 ---- */	// もし、内包されているビューから継承できればスマート。
	
	NSMutableDictionary *attrs = [[[NSMutableDictionary alloc] init] autorelease];
	
	[attrs addEntriesFromDictionary:[[self attributedStringValue] attributesAtIndex:0 effectiveRange:NULL]];
	
	[attrs setObject:[NSFont controlContentFontOfSize:fontSize] forKey:NSFontAttributeName];
	
	// 反転表示されていて、かつ自分が表示されているビューにフォーカスが当たっていれば、文字色を白にしなければならない。
	if ( [self isHighlighted] && [[NSApp keyWindow] firstResponder] == parenticView ) {
		[attrs setObject:[NSColor alternateSelectedControlTextColor] forKey:NSForegroundColorAttributeName];
	}
	
	
	/* ---- 自分（セル）の幅から、日付の表示形式を決定 ---- */
	
	#define NUMBER_OF_DATE_STYLES 4
	#define NUMBER_OF_TIME_STYLES 2
	
	NSDateFormatterStyle dateStyles[NUMBER_OF_DATE_STYLES] = {
		NSDateFormatterFullStyle, // 曜日含む。Mail.app では使われていない。
		NSDateFormatterLongStyle,
		NSDateFormatterMediumStyle,
		NSDateFormatterShortStyle,
	};
	
	NSDateFormatterStyle timeStyles[NUMBER_OF_TIME_STYLES] = {
		NSDateFormatterShortStyle,
		NSDateFormatterNoStyle
	};
	
	float requiredWidth = cellFrame.size.width - (HORIZONTAL_MARGIN * 2); // 現在のセルの幅（ − 左右のマージン）
	
	int i, j;
	NSString *timeDateString, *dateString, *timeString;
	NSDateFormatterStyle dateStyle, timeStyle;
	
	float dateTimeSpacing = (HORIZONTAL_MARGIN * 2); // 日付と時刻があまりに接近すると意味がないので、最低限の幅。
	
	for ( i = 0; i < NUMBER_OF_TIME_STYLES; i++ ) {
	
		timeStyle = timeStyles[i];
				
		for ( j = 0; j < NUMBER_OF_DATE_STYLES; j++ ) {
		
			dateStyle = dateStyles[j];
			
			[formatter setDateStyle:dateStyle];
			[formatter setTimeStyle:timeStyle];
			
			timeDateString = [formatter stringFromDate:testDate];
			
			if ( [timeDateString sizeWithAttributes:attrs].width + dateTimeSpacing <= requiredWidth ) break;
			
			timeDateString = nil;
		
		}
		
		if ( timeDateString != nil && [timeDateString sizeWithAttributes:attrs].width + dateTimeSpacing <= requiredWidth ) break;
		
		timeDateString = nil;
		
	}
	
	/* ---- 表示する文字列の準備・相対日付（「今日」「昨日」など）---- */
	
	// メソッド dayOfCommonEra（１世紀からの日数？　ちょっと大袈裟な気もする）で、ややこしい時刻情報を飛ばし、純粋に日付だけで計算する
	int dayOfCommonEra = [[date dateWithCalendarFormat:nil timeZone:nil] dayOfCommonEra];
	int todayOfCommonEra = [[NSCalendarDate calendarDate] dayOfCommonEra];
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	
	if ( dayOfCommonEra == todayOfCommonEra ) dateString = [[[defaults arrayForKey:NSThisDayDesignations] objectAtIndex:0] capitalizedString]; // ユーザーの言語で「今日」
		
	else if ( dayOfCommonEra == ( todayOfCommonEra - 1 ) ) dateString = [[[defaults arrayForKey:NSPriorDayDesignations] objectAtIndex:0] capitalizedString]; // 「昨日」
	else { // 通常表示
	
		[formatter setDateStyle:dateStyle];
		[formatter setTimeStyle:NSDateFormatterNoStyle];
		
		dateString = [formatter stringFromDate:date];
		
	}
	
	[formatter setDateStyle:NSDateFormatterNoStyle];
	[formatter setTimeStyle:timeStyle];
	
	timeString = [formatter stringFromDate:date];
	
	/* ---- 描画 ---- */

	NSRect pathRect;
	
    pathRect.origin.x = cellFrame.origin.x + HORIZONTAL_MARGIN;
	pathRect.origin.y = cellFrame.origin.y;
    pathRect.size.width = cellFrame.size.width 
                - (pathRect.origin.x - cellFrame.origin.x) - (HORIZONTAL_MARGIN * 2);
    pathRect.size.height = cellFrame.size.height;
    
	[dateString drawInRect:pathRect withAttributes:attrs];
	
	// 時刻にだけ「右寄せ」アトリビュートを追加する
	NSMutableParagraphStyle *pStyle = [[NSMutableParagraphStyle defaultParagraphStyle] mutableCopy];
	[pStyle setAlignment:NSRightTextAlignment];
	[attrs setObject:pStyle forKey:NSParagraphStyleAttributeName];
    
	[timeString drawInRect:pathRect withAttributes:attrs];
	
	[pStyle release];

}

- (void)setPlaceholderString:(NSString *)string {} // （たぶん）objectValue が空のとき、NSTextFieldView じゃないのに RBSplitView が呼んできてエラーになるので。


@end
